@@ -1,5 +1,5 @@
 /* obsolete.c --- Obsolete functions kept around for backwards compatibility.
- * Copyright (C) 2002, 2003, 2004  Simon Josefsson
+ * Copyright (C) 2002, 2003, 2004, 2005  Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -13,16 +13,15 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * License along with GNU SASL Library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License License along with GNU SASL Library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
 #include "internal.h"
 
-#ifndef GSASL_NO_OBSOLETE
-
 /**
  * gsasl_client_listmech:
  * @ctx: libgsasl handle.
@@ -1607,9 +1606,9 @@ char *
 gsasl_stringprep_saslprep (const char *in, int *stringprep_rc)
 {
   char *out = NULL;
+#if WITH_SASLPREP
   int rc;
 
-#if WITH_SASLPREP
   rc = stringprep_profile (in, &out, "SASLprep", 0);
   if (stringprep_rc)
     *stringprep_rc = rc;
@@ -1641,9 +1640,9 @@ char *
 gsasl_stringprep_trace (const char *in, int *stringprep_rc)
 {
   char *out = NULL;
+#if WITH_SASLPREP
   int rc;
 
-#if WITH_SASLPREP
   rc = stringprep_profile (in, &out, "trace", 0);
   if (stringprep_rc)
     *stringprep_rc = rc;
@@ -1654,6 +1653,83 @@ gsasl_stringprep_trace (const char *in, int *stringprep_rc)
   return out;
 }
 
+/**
+ * gsasl_md5pwd_get_password:
+ * @filename: filename of file containing passwords.
+ * @username: username string.
+ * @key: output character array.
+ * @keylen: input maximum size of output character array, on output
+ * contains actual length of output array.
+ *
+ * Retrieve password for user from specified file.  To find out how
+ * large the output array must be, call this function with out=NULL.
+ *
+ * The file should be on the UoW "MD5 Based Authentication" format,
+ * which means it is in text format with comments denoted by # first
+ * on the line, with user entries looking as username\tpassword.  This
+ * function removes \r and \n at the end of lines before processing.
+ *
+ * Return value: Return GSASL_OK if output buffer contains the
+ * password, GSASL_AUTHENTICATION_ERROR if the user could not be
+ * found, or other error code.
+ *
+ * Deprecated: Use gsasl_simple_getpass() instead.
+ **/
+int
+gsasl_md5pwd_get_password (const char *filename,
+			   const char *username, char *key, size_t * keylen)
+{
+  char matchbuf[BUFSIZ];
+  char line[BUFSIZ];
+  FILE *fh;
+
+  fh = fopen (filename, "r");
+  if (fh == NULL)
+    return GSASL_FOPEN_ERROR;
+
+  sprintf (matchbuf, "%s\t", username);
+
+  while (!feof (fh))
+    {
+      if (fgets (line, BUFSIZ, fh) == NULL)
+	break;
+
+      if (line[0] == '#')
+	continue;
+
+      while (strlen (line) > 0 && (line[strlen (line) - 1] == '\n' ||
+				   line[strlen (line) - 1] == '\r'))
+	line[strlen (line) - 1] = '\0';
+
+      if (strlen (line) <= strlen (matchbuf))
+	continue;
+
+      if (strncmp (matchbuf, line, strlen (matchbuf)) == 0)
+	{
+	  if (*keylen < strlen (line) - strlen (matchbuf))
+	    {
+	      fclose (fh);
+	      return GSASL_TOO_SMALL_BUFFER;
+	    }
+
+	  *keylen = strlen (line) - strlen (matchbuf);
+
+	  if (key)
+	    memcpy (key, &line[strlen (matchbuf)], *keylen);
+
+	  fclose (fh);
+
+	  return GSASL_OK;
+	}
+    }
+
+  if (fclose (fh) != 0)
+    return GSASL_FCLOSE_ERROR;
+
+  return GSASL_AUTHENTICATION_ERROR;
+}
+
+
 /*
  * Copyright (c) 1996-1999 by Internet Software Consortium.
  *
@@ -1926,4 +2002,3 @@ gsasl_base64_decode (char const *src, char *target, size_t targsize)
 
   return (tarindex);
 }
-#endif
