@@ -42,11 +42,18 @@ writeln (const char *str)
 
   if (sockfd)
     {
-      size_t len;
+      ssize_t len;
 
 #ifdef HAVE_LIBGNUTLS
       if (using_tls)
-	len = gnutls_record_send (session, str, strlen (str));
+	{
+	  /* GnuTLS < 1.2.9 cannot handle data != NULL && count == 0,
+	     it will return an error. */
+	  if (len > 0)
+	    len = gnutls_record_send (session, str, strlen (str));
+	  else
+	    len = 0;
+	}
       else
 #endif
 	len = write (sockfd, str, strlen (str));
